@@ -75,6 +75,9 @@ build_c_helpers_dict(void)
     declmethod(get_list_private_data);
     declmethod(set_list_private_data);
     declmethod(reset_list_private_data);
+    declmethod(get_pyobject_private_data);
+    declmethod(set_pyobject_private_data);
+    declmethod(reset_pyobject_private_data);
     declmethod(xxgemm);
     declmethod(xxgemv);
     declmethod(xxdot);
