@@ -107,7 +107,7 @@ class PyCallWrapper(object):
         wrapty = Type.function(pyobj, [pyobj, pyobj, pyobj])
         wrapper = self.module.add_function(wrapty, name=wrapname)
 
-        builder = Builder.new(wrapper.append_basic_block('entry'))
+        builder = Builder(wrapper.append_basic_block('entry'))
 
         # - `closure` will receive the `self` pointer stored in the
         #   PyCFunction object (see _dynfunc.c)
