@@ -92,7 +92,7 @@ def from_dtype(dtype):
         except KeyError:
             if dtype.char in 'SU':
                 return _from_str_dtype(dtype)
-            if dtype.char in 'mM' and npdatetime.NPDATETIME_SUPPORTED:
+            if dtype.char in 'mM':
                 return _from_datetime_dtype(dtype)
             if dtype.char in 'V':
                 subtype = from_dtype(dtype.subdtype[0])
@@ -346,23 +346,8 @@ def ufunc_find_matching_loop(ufunc, arg_types):
     return None
 
 
-if numpy.__version__ <= '1.7':
-    # Numpy 1.6 and below does not have the isalignedstruct attribute
-    # for dtype.  We implement the behavior of the function here.
-    def _is_aligned_struct(struct):
-        for (dtype, offset) in struct.fields.values():
-            # Make sure all attributes can only appear at offsets equal
-            # to the alignment of the attribute dtype.
-            if offset % dtype.alignment:
-                return False
-            if struct.itemsize % dtype.alignment:
-                return False
-        return True
-
-else:
-    # NumPy 1.7 and above uses the isalignedstruct property.
-    def _is_aligned_struct(struct):
-        return struct.isalignedstruct
+def _is_aligned_struct(struct):
+    return struct.isalignedstruct
 
 
 def from_struct_dtype(dtype):
