@@ -5,4 +5,4 @@ A subpackage hosting Numba IR rewrite passes.
 from .registry import register_rewrite, rewrite_registry, Rewrite
 
 # Register various built-in rewrite passes
-from . import static_getitem, static_raise
+from . import static_getitem, static_raise, macros
