@@ -7,7 +7,7 @@ from llvmlite import ir
 from .. import cgutils, numpy_support, types
 from ..pythonapi import box, unbox, reflect, NativeValue
 
-from . import listobj
+from . import listobj, setobj
 
 
 #
@@ -77,8 +77,7 @@ def unbox_float(typ, obj, c):
 
 @box(types.Complex)
 def box_complex(typ, val, c):
-    cmplxcls = c.context.make_complex(typ)
-    cval = cmplxcls(c.context, c.builder, value=val)
+    cval = c.context.make_complex(c.builder, typ, value=val)
 
     if typ == types.complex64:
         freal = c.builder.fpext(cval.real, c.pyapi.double)
@@ -90,8 +89,8 @@ def box_complex(typ, val, c):
 
 @unbox(types.Complex)
 def unbox_complex(typ, obj, c):
-    c128cls = c.context.make_complex(types.complex128)
-    c128 = c128cls(c.context, c.builder)
+    # First unbox to complex128, since that's what CPython gives us
+    c128 = c.context.make_complex(c.builder, types.complex128)
     ok = c.pyapi.complex_adaptor(obj, c128._getpointer())
     failed = cgutils.is_false(c.builder, ok)
 
@@ -100,8 +99,8 @@ def unbox_complex(typ, obj, c):
                                "conversion to %s failed" % (typ,))
 
     if typ == types.complex64:
-        cplxcls = c.context.make_complex(typ)
-        cplx = cplxcls(c.context, c.builder)
+        # Downcast to complex64 if necessary
+        cplx = c.context.make_complex(c.builder, typ)
         cplx.real = c.context.cast(c.builder, c128.real,
                                    types.float64, types.float32)
         cplx.imag = c.context.cast(c.builder, c128.imag,
@@ -280,7 +279,7 @@ def unbox_slice(typ, obj, c):
     """
     from . import slicing
     ok, start, stop, step = c.pyapi.slice_as_ints(obj)
-    sli = slicing.make_slice(c.context, c.builder, typ)
+    sli = c.context.make_helper(c.builder, typ)
     sli.start = start
     sli.stop = stop
     sli.step = step
@@ -310,9 +309,9 @@ def box_array(typ, val, c):
         return parent
 
 @box(types.SmartArrayType)
-def box_smart_array(type, value, c):
+def box_smart_array(typ, value, c):
     # First build a Numpy array object, then wrap it in a SmartArray
-    a = cgutils.create_struct_proxy(type)(c.context, c.builder, value=value)
+    a = c.context.make_helper(c.builder, typ, value=value)
     # if 'parent' is set, we are re-boxing an object, so use the same logic
     # as reflect.
     obj = a.parent
@@ -328,8 +327,8 @@ def box_smart_array(type, value, c):
                     c.pyapi.raise_object()
         with otherwise:
             # box into a new array:
-            classobj = c.pyapi.unserialize(c.pyapi.serialize_object(type.pyclass))
-            arrayobj = c.box(type.as_array, a.data)
+            classobj = c.pyapi.unserialize(c.pyapi.serialize_object(typ.pyclass))
+            arrayobj = c.box(typ.as_array, a.data)
             # Adopt arrayobj rather than copying it.
             false = c.pyapi.bool_from_bool(cgutils.false_bit)
             obj = c.pyapi.call_function_objargs(classobj, (arrayobj,false))
@@ -390,12 +389,12 @@ def unbox_array(typ, obj, c):
 
 
 @unbox(types.SmartArrayType)
-def unbox_smart_array(type, obj, c):
-    a = cgutils.create_struct_proxy(type)(c.context, c.builder)
+def unbox_smart_array(typ, obj, c):
+    a = c.context.make_helper(c.builder, typ)
     arr = c.pyapi.call_method(obj, "host")
     with c.builder.if_else(cgutils.is_not_null(c.builder, arr)) as (success, failure):
         with success:
-            a.data = c.unbox(type.as_array, arr).value
+            a.data = c.unbox(typ.as_array, arr).value
             a.parent = obj
             c.pyapi.decref(arr)
         with failure:
@@ -405,8 +404,8 @@ def unbox_smart_array(type, obj, c):
 
 
 @reflect(types.SmartArrayType)
-def reflect_smart_array(type, value, c):
-    a = cgutils.create_struct_proxy(type)(c.context, c.builder, value)
+def reflect_smart_array(typ, value, c):
+    a = c.context.make_helper(c.builder, typ, value)
     arr = a.parent
     retn = c.pyapi.call_method(arr, "host_changed")
     with c.builder.if_else(cgutils.is_not_null(c.builder, retn)) as (success, failure):
@@ -662,6 +661,178 @@ def reflect_list(typ, val, c):
         list.set_dirty(False)
 
 
+def _python_set_to_native(typ, obj, c, size, setptr, errorptr):
+    """
+    Construct a new native set from a Python set.
+    """
+    # Allocate a new native set
+    ok, inst = setobj.SetInstance.allocate_ex(c.context, c.builder, typ, size)
+    with c.builder.if_else(ok, likely=True) as (if_ok, if_not_ok):
+        with if_ok:
+            # Traverse Python set and unbox objects into native set
+            typobjptr = cgutils.alloca_once_value(c.builder,
+                                                  ir.Constant(c.pyapi.pyobj, None))
+
+            with c.pyapi.set_iterate(obj) as loop:
+                itemobj = loop.value
+                # Mandate that objects all have the same exact type
+                typobj = c.pyapi.get_type(itemobj)
+                expected_typobj = c.builder.load(typobjptr)
+
+                with c.builder.if_else(
+                    cgutils.is_null(c.builder, expected_typobj),
+                    likely=False) as (if_first, if_not_first):
+                    with if_first:
+                        # First iteration => store item type
+                        c.builder.store(typobj, typobjptr)
+                    with if_not_first:
+                        # Otherwise, check item type
+                        type_mismatch = c.builder.icmp_signed('!=', typobj,
+                                                              expected_typobj)
+                        with c.builder.if_then(type_mismatch, likely=False):
+                            c.builder.store(cgutils.true_bit, errorptr)
+                            c.pyapi.err_set_string("PyExc_TypeError",
+                                                   "can't unbox heterogenous set")
+                            loop.do_break()
+
+                # XXX we don't call native cleanup for each set element,
+                # since that would require keeping track
+                # of which unboxings have been successful.
+                native = c.unbox(typ.dtype, itemobj)
+                with c.builder.if_then(native.is_error, likely=False):
+                    c.builder.store(cgutils.true_bit, errorptr)
+                inst.add(native.value, do_resize=False)
+
+            if typ.reflected:
+                inst.parent = obj
+            # Associate meminfo pointer with the Python object for later reuse.
+            with c.builder.if_then(c.builder.not_(c.builder.load(errorptr)),
+                                   likely=False):
+                c.pyapi.object_set_private_data(obj, inst.meminfo)
+            inst.set_dirty(False)
+            c.builder.store(inst.value, setptr)
+
+        with if_not_ok:
+            c.builder.store(cgutils.true_bit, errorptr)
+
+    # If an error occurred, drop the whole native set
+    with c.builder.if_then(c.builder.load(errorptr)):
+        c.context.nrt_decref(c.builder, typ, inst.value)
+
+
+@unbox(types.Set)
+def unbox_set(typ, obj, c):
+    """
+    Convert set *obj* to a native set.
+
+    If set was previously unboxed, we reuse the existing native set
+    to ensure consistency.
+    """
+    size = c.pyapi.set_size(obj)
+
+    errorptr = cgutils.alloca_once_value(c.builder, cgutils.false_bit)
+    setptr = cgutils.alloca_once(c.builder, c.context.get_value_type(typ))
+
+    # See if the set was previously unboxed, if so, re-use the meminfo.
+    ptr = c.pyapi.object_get_private_data(obj)
+
+    with c.builder.if_else(cgutils.is_not_null(c.builder, ptr)) \
+        as (has_meminfo, otherwise):
+
+        with has_meminfo:
+            # Set was previously unboxed => reuse meminfo
+            inst = setobj.SetInstance.from_meminfo(c.context, c.builder, typ, ptr)
+            if typ.reflected:
+                inst.parent = obj
+            c.builder.store(inst.value, setptr)
+
+        with otherwise:
+            _python_set_to_native(typ, obj, c, size, setptr, errorptr)
+
+    def cleanup():
+        # Clean up the associated pointer, as the meminfo is now invalid.
+        c.pyapi.object_reset_private_data(obj)
+
+    return NativeValue(c.builder.load(setptr),
+                       is_error=c.builder.load(errorptr),
+                       cleanup=cleanup)
+
+
+def _native_set_to_python_list(typ, payload, c):
+    """
+    Create a Python list from a native set's items.
+    """
+    nitems = payload.used
+    listobj = c.pyapi.list_new(nitems)
+    ok = cgutils.is_not_null(c.builder, listobj)
+    with c.builder.if_then(ok, likely=True):
+        index = cgutils.alloca_once_value(c.builder,
+                                          ir.Constant(nitems.type, 0))
+        with payload._iterate() as loop:
+            i = c.builder.load(index)
+            item = loop.entry.key
+            itemobj = c.box(typ.dtype, item)
+            c.pyapi.list_setitem(listobj, i, itemobj)
+            i = c.builder.add(i, ir.Constant(i.type, 1))
+            c.builder.store(i, index)
+
+    return ok, listobj
+
+
+@box(types.Set)
+def box_set(typ, val, c):
+    """
+    Convert native set *val* to a set object.
+    """
+    inst = setobj.SetInstance(c.context, c.builder, typ, val)
+    obj = inst.parent
+    res = cgutils.alloca_once_value(c.builder, obj)
+
+    with c.builder.if_else(cgutils.is_not_null(c.builder, obj)) as (has_parent, otherwise):
+        with has_parent:
+            # Set is actually reflected => return the original object
+            # (note not all set instances whose *type* is reflected are
+            #  actually reflected; see numba.tests.test_sets for an example)
+            c.pyapi.incref(obj)
+
+        with otherwise:
+            # Build a new Python list and then create a set from that
+            payload = inst.payload
+            ok, listobj = _native_set_to_python_list(typ, payload, c)
+            with c.builder.if_then(ok, likely=True):
+                obj = c.pyapi.set_new(listobj)
+                c.pyapi.decref(listobj)
+                c.builder.store(obj, res)
+
+    # Steal NRT ref
+    c.context.nrt_decref(c.builder, typ, val)
+    return c.builder.load(res)
+
+@reflect(types.Set)
+def reflect_set(typ, val, c):
+    """
+    Reflect the native set's contents into the Python object.
+    """
+    if not typ.reflected:
+        return
+    inst = setobj.SetInstance(c.context, c.builder, typ, val)
+    payload = inst.payload
+
+    with c.builder.if_then(payload.dirty, likely=False):
+        obj = inst.parent
+        # XXX errors are not dealt with below
+        c.pyapi.set_clear(obj)
+
+        # Build a new Python list and then update the set with that
+        ok, listobj = _native_set_to_python_list(typ, payload, c)
+        with c.builder.if_then(ok, likely=True):
+            c.pyapi.set_update(obj, listobj)
+            c.pyapi.decref(listobj)
+
+        # Mark the set clean, in case it is reflected twice
+        inst.set_dirty(False)
+
+
 #
 # Other types
 #
