@@ -5,7 +5,8 @@ import sys
 import numpy as np
 
 from numba import cffi_support
-from numba.tests.support import static_temp_directory
+from numba.tests.support import import_dynamic, temp_directory
+from numba.types import complex128
 
 
 def load_inline_module():
@@ -44,15 +45,24 @@ def load_ool_module():
     """
     from cffi import FFI
 
-    defs = """
-    double sin(double x);
-    double cos(double x);
-    void vsSin(int n, float* x, float* y);
-    void vdSin(int n, double* x, double* y);
-    int foo(int a, int b, int c);
+    numba_complex = """
+    typedef struct _numba_complex {
+        double real;
+        double imag;
+    } numba_complex;
     """
 
-    source = """
+    defs = numba_complex + """
+    double sin(double x);
+    double cos(double x);
+    int foo(int a, int b, int c);
+    void vsSin(int n, float* x, float* y);
+    void vdSin(int n, double* x, double* y);
+    void vector_real(numba_complex *c, double *real, int n);
+    void vector_imag(numba_complex *c, double *imag, int n);
+    """
+
+    source = numba_complex + """
     static int foo(int a, int b, int c)
     {
         return a + b * c;
@@ -69,35 +79,68 @@ def load_ool_module():
         for (i=0; i<n; i++)
             y[i] = sin(x[i]);
     }
+
+    static void vector_real(numba_complex *c, double *real, int n) {
+        int i;
+        for (i = 0; i < n; i++)
+            real[i] = c[i].real;
+    }
+
+    static void vector_imag(numba_complex *c, double *imag, int n) {
+        int i;
+        for (i = 0; i < n; i++)
+            imag[i] = c[i].imag;
+    }
     """
 
     ffi = FFI()
     ffi.set_source('cffi_usecases_ool', source)
     ffi.cdef(defs, override=True)
-    tmpdir = static_temp_directory('test_cffi')
+    tmpdir = temp_directory('test_cffi')
     ffi.compile(tmpdir=tmpdir)
     sys.path.append(tmpdir)
     try:
-        import cffi_usecases_ool as mod
+        mod = import_dynamic('cffi_usecases_ool')
         cffi_support.register_module(mod)
+        cffi_support.register_type(mod.ffi.typeof('struct _numba_complex'),
+                                   complex128)
         return mod.ffi, mod
     finally:
         sys.path.remove(tmpdir)
 
 
-if cffi_support.SUPPORTED:
-    ffi, dll = load_inline_module()
-    cffi_sin = dll._numba_test_sin
-    cffi_cos = dll._numba_test_cos
-    del dll
+def init():
+    """
+    Initialize module globals.  This can invoke external utilities, hence not
+    being executed implicitly at module import.
+    """
+    global ffi, cffi_sin, cffi_cos
 
-    ffi_ool, mod = load_ool_module()
-    cffi_sin_ool = mod.lib.sin
-    cffi_cos_ool = mod.lib.cos
-    cffi_foo = mod.lib.foo
-    vsSin = mod.lib.vsSin
-    vdSin = mod.lib.vdSin
-    del mod
+    if ffi is None:
+        ffi, dll = load_inline_module()
+        cffi_sin = dll._numba_test_sin
+        cffi_cos = dll._numba_test_cos
+        del dll
+
+def init_ool():
+    """
+    Same as init() for OOL mode.
+    """
+    global ffi_ool, cffi_sin_ool, cffi_cos_ool, cffi_foo, vsSin, vdSin
+    global vector_real, vector_imag
+
+    if ffi_ool is None:
+        ffi_ool, mod = load_ool_module()
+        cffi_sin_ool = mod.lib.sin
+        cffi_cos_ool = mod.lib.cos
+        cffi_foo = mod.lib.foo
+        vsSin = mod.lib.vsSin
+        vdSin = mod.lib.vdSin
+        vector_real = mod.lib.vector_real
+        vector_imag = mod.lib.vector_imag
+        del mod
+
+ffi = ffi_ool = None
 
 
 def use_cffi_sin(x):
@@ -125,12 +168,17 @@ def use_user_defined_symbols():
 # (cffi_usecases_ool.ffi is a CompiledFFI object) so we use both in these
 # functions.
 
-def vector_sin_float32(x):
-    y = np.empty_like(x)
+def vector_sin_float32(x, y):
     vsSin(len(x), ffi.from_buffer(x), ffi_ool.from_buffer(y))
-    return y
 
-def vector_sin_float64(x):
-    y = np.empty_like(x)
+def vector_sin_float64(x, y):
     vdSin(len(x), ffi.from_buffer(x), ffi_ool.from_buffer(y))
-    return y
+
+
+# For testing pointer to structs from buffers
+
+def vector_extract_real(x, y):
+    vector_real(ffi.from_buffer(x), ffi.from_buffer(y), len(x))
+
+def vector_extract_imag(x, y):
+    vector_imag(ffi.from_buffer(x), ffi.from_buffer(y), len(x))
