@@ -9,6 +9,7 @@ import gc
 import math
 import os
 import shutil
+import subprocess
 import sys
 import tempfile
 import time
@@ -30,10 +31,8 @@ force_pyobj_flags.set("force_pyobject")
 
 no_pyobj_flags = Flags()
 
-
-is_on_numpy_16 = numpy_support.version == (1, 6)
-skip_on_numpy_16 = unittest.skipIf(is_on_numpy_16,
-                                   "test requires Numpy 1.7 or later")
+nrt_flags = Flags()
+nrt_flags.set("nrt")
 
 
 tag = testing.make_tag_decorator(['important'])
@@ -104,7 +103,7 @@ class TestCase(unittest.TestCase):
         """
         A context manager that asserts the given objects have the
         same reference counts before and after executing the
-        enclosed blocks.
+        enclosed block.
         """
         old_refcounts = [sys.getrefcount(x) for x in objects]
         yield
@@ -114,6 +113,25 @@ class TestCase(unittest.TestCase):
                 self.fail("Refcount changed from %d to %d for object: %r"
                           % (old, new, obj))
 
+    @contextlib.contextmanager
+    def assertNoNRTLeak(self):
+        """
+        A context manager that asserts no NRT leak was created during
+        the execution of the enclosed block.
+        """
+        old = rtsys.get_allocation_stats()
+        yield
+        new = rtsys.get_allocation_stats()
+        total_alloc = new.alloc - old.alloc
+        total_free = new.free - old.free
+        total_mi_alloc = new.mi_alloc - old.mi_alloc
+        total_mi_free = new.mi_free - old.mi_free
+        self.assertEqual(total_alloc, total_free,
+                         "number of data allocs != number of data frees")
+        self.assertEqual(total_mi_alloc, total_mi_free,
+                         "number of meminfo allocs != number of meminfo frees")
+
+
     _exact_typesets = [(bool, np.bool_), utils.INT_TYPES, (str,), (np.integer,), (utils.text_type), ]
     _approx_typesets = [(float,), (complex,), (np.inexact)]
     _sequence_typesets = [(tuple, list)]
@@ -382,7 +400,7 @@ def compile_function(name, code, globs):
     Given a *code* string, compile it with globals *globs* and return
     the function named *name*.
     """
-    co = compile(code, "<string>", "exec")
+    co = compile(code.rstrip(), "<string>", "single")
     ns = {}
     eval(co, globs, ns)
     return ns[name]
@@ -550,3 +568,36 @@ class MemoryLeakMixin(MemoryLeak):
     def tearDown(self):
         super(MemoryLeakMixin, self).tearDown()
         self.memory_leak_teardown()
+
+
+@contextlib.contextmanager
+def forbid_codegen():
+    """
+    Forbid LLVM code generation during the execution of the context
+    manager's enclosed block.
+
+    If code generation is invoked, a RuntimeError is raised.
+    """
+    from numba.targets import codegen
+    patchpoints = ['CodeLibrary._finalize_final_module']
+
+    old = {}
+    def fail(*args, **kwargs):
+        raise RuntimeError("codegen forbidden by test case")
+    try:
+        # XXX use the mock library instead?
+        for name in patchpoints:
+            parts = name.split('.')
+            obj = codegen
+            for attrname in parts[:-1]:
+                obj = getattr(obj, attrname)
+            attrname = parts[-1]
+            value = getattr(obj, attrname)
+            assert callable(value), ("%r should be callable" % name)
+            old[obj, attrname] = value
+            setattr(obj, attrname, fail)
+        yield
+    finally:
+        for (obj, attrname), value in old.items():
+            setattr(obj, attrname, value)
+
