@@ -4,6 +4,7 @@ import errno
 import imp
 import os
 import shutil
+import stat
 import subprocess
 import sys
 import tempfile
@@ -13,7 +14,7 @@ import warnings
 import numpy as np
 
 from numba import unittest_support as unittest
-from numba import utils, vectorize, jit, generated_jit, types
+from numba import utils, vectorize, jit, generated_jit, types, appdirs
 from numba.errors import NumbaWarning
 from .support import TestCase, tag, temp_directory, import_dynamic
 
@@ -539,7 +540,7 @@ class TestCache(TestCase):
         return dict((fn, os.path.getmtime(os.path.join(self.cache_dir, fn)))
                     for fn in sorted(self.cache_contents()))
 
-    def check_cache(self, n):
+    def check_pycache(self, n):
         c = self.cache_contents()
         self.assertEqual(len(c), n, c)
 
@@ -575,43 +576,53 @@ class TestCache(TestCase):
                                  % (popen.returncode, err.decode()))
 
     def check_module(self, mod):
-        self.check_cache(0)
+        self.check_pycache(0)
         f = mod.add_usecase
         self.assertPreciseEqual(f(2, 3), 6)
-        self.check_cache(2)  # 1 index, 1 data
+        self.check_pycache(2)  # 1 index, 1 data
         self.assertPreciseEqual(f(2.5, 3), 6.5)
-        self.check_cache(3)  # 1 index, 2 data
+        self.check_pycache(3)  # 1 index, 2 data
 
         f = mod.add_objmode_usecase
         self.assertPreciseEqual(f(2, 3), 6)
-        self.check_cache(5)  # 2 index, 3 data
+        self.check_pycache(5)  # 2 index, 3 data
         self.assertPreciseEqual(f(2.5, 3), 6.5)
-        self.check_cache(6)  # 2 index, 4 data
+        self.check_pycache(6)  # 2 index, 4 data
+
+    def check_hits(self, func, hits, misses=None):
+        st = func.stats
+        self.assertEqual(sum(st.cache_hits.values()), hits, st.cache_hits)
+        if misses is not None:
+            self.assertEqual(sum(st.cache_misses.values()), misses,
+                             st.cache_misses)
 
     @tag('important')
     def test_caching(self):
-        self.check_cache(0)
+        self.check_pycache(0)
         mod = self.import_module()
-        self.check_cache(0)
+        self.check_pycache(0)
 
         f = mod.add_usecase
         self.assertPreciseEqual(f(2, 3), 6)
-        self.check_cache(2)  # 1 index, 1 data
+        self.check_pycache(2)  # 1 index, 1 data
         self.assertPreciseEqual(f(2.5, 3), 6.5)
-        self.check_cache(3)  # 1 index, 2 data
+        self.check_pycache(3)  # 1 index, 2 data
+        self.check_hits(f, 0, 2)
 
         f = mod.add_objmode_usecase
         self.assertPreciseEqual(f(2, 3), 6)
-        self.check_cache(5)  # 2 index, 3 data
+        self.check_pycache(5)  # 2 index, 3 data
         self.assertPreciseEqual(f(2.5, 3), 6.5)
-        self.check_cache(6)  # 2 index, 4 data
+        self.check_pycache(6)  # 2 index, 4 data
+        self.check_hits(f, 0, 2)
 
         f = mod.record_return
         rec = f(mod.aligned_arr, 1)
         self.assertPreciseEqual(tuple(rec), (2, 43.5))
         rec = f(mod.packed_arr, 1)
         self.assertPreciseEqual(tuple(rec), (2, 43.5))
-        self.check_cache(9)  # 3 index, 6 data
+        self.check_pycache(9)  # 3 index, 6 data
+        self.check_hits(f, 0, 2)
 
         f = mod.generated_usecase
         self.assertPreciseEqual(f(3, 2), 1)
@@ -624,42 +635,42 @@ class TestCache(TestCase):
         # Caching inner then outer function is ok
         mod = self.import_module()
         self.assertPreciseEqual(mod.inner(3, 2), 6)
-        self.check_cache(2)  # 1 index, 1 data
+        self.check_pycache(2)  # 1 index, 1 data
         # Uncached outer function shouldn't fail (issue #1603)
         f = mod.outer_uncached
         self.assertPreciseEqual(f(3, 2), 2)
-        self.check_cache(2)  # 1 index, 1 data
+        self.check_pycache(2)  # 1 index, 1 data
         mod = self.import_module()
         f = mod.outer_uncached
         self.assertPreciseEqual(f(3, 2), 2)
-        self.check_cache(2)  # 1 index, 1 data
+        self.check_pycache(2)  # 1 index, 1 data
         # Cached outer will create new cache entries
         f = mod.outer
         self.assertPreciseEqual(f(3, 2), 2)
-        self.check_cache(4)  # 2 index, 2 data
+        self.check_pycache(4)  # 2 index, 2 data
         self.assertPreciseEqual(f(3.5, 2), 2.5)
-        self.check_cache(6)  # 2 index, 4 data
+        self.check_pycache(6)  # 2 index, 4 data
 
     def test_outer_then_inner(self):
         # Caching outer then inner function is ok
         mod = self.import_module()
         self.assertPreciseEqual(mod.outer(3, 2), 2)
-        self.check_cache(4)  # 2 index, 2 data
+        self.check_pycache(4)  # 2 index, 2 data
         self.assertPreciseEqual(mod.outer_uncached(3, 2), 2)
-        self.check_cache(4)  # same
+        self.check_pycache(4)  # same
         mod = self.import_module()
         f = mod.inner
         self.assertPreciseEqual(f(3, 2), 6)
-        self.check_cache(4)  # same
+        self.check_pycache(4)  # same
         self.assertPreciseEqual(f(3.5, 2), 6.5)
-        self.check_cache(5)  # 2 index, 3 data
+        self.check_pycache(5)  # 2 index, 3 data
 
     def test_no_caching(self):
         mod = self.import_module()
 
         f = mod.add_nocache_usecase
         self.assertPreciseEqual(f(2, 3), 6)
-        self.check_cache(0)
+        self.check_pycache(0)
 
     def test_looplifted(self):
         # Loop-lifted functions can't be cached and raise a warning
@@ -670,7 +681,7 @@ class TestCache(TestCase):
 
             f = mod.looplifted
             self.assertPreciseEqual(f(4), 6)
-            self.check_cache(0)
+            self.check_pycache(0)
 
         self.assertEqual(len(w), 1)
         self.assertEqual(str(w[0].message),
@@ -687,7 +698,7 @@ class TestCache(TestCase):
 
             f = mod.use_c_sin
             self.assertPreciseEqual(f(0.0), 0.0)
-            self.check_cache(0)
+            self.check_pycache(0)
 
         self.assertEqual(len(w), 1)
         self.assertIn('Cannot cache compiled function "use_c_sin"',
@@ -703,7 +714,7 @@ class TestCache(TestCase):
             self.assertPreciseEqual(f(3), 6)
             f = mod.closure2
             self.assertPreciseEqual(f(3), 8)
-            self.check_cache(0)
+            self.check_pycache(0)
 
         self.assertEqual(len(w), 2)
         for item in w:
@@ -713,6 +724,7 @@ class TestCache(TestCase):
     def test_cache_reuse(self):
         mod = self.import_module()
         mod.add_usecase(2, 3)
+        mod.add_usecase(2.5, 3.5)
         mod.add_objmode_usecase(2, 3)
         mod.outer_uncached(2, 3)
         mod.outer(2, 3)
@@ -720,11 +732,19 @@ class TestCache(TestCase):
         mod.record_return(mod.aligned_arr, 1)
         mod.generated_usecase(2, 3)
         mtimes = self.get_cache_mtimes()
+        # Two signatures compiled
+        self.check_hits(mod.add_usecase, 0, 2)
 
         mod2 = self.import_module()
         self.assertIsNot(mod, mod2)
-        mod.add_usecase(2, 3)
-        mod.add_objmode_usecase(2, 3)
+        f = mod2.add_usecase
+        f(2, 3)
+        self.check_hits(f, 1, 0)
+        f(2.5, 3.5)
+        self.check_hits(f, 2, 0)
+        f = mod2.add_objmode_usecase
+        f(2, 3)
+        self.check_hits(f, 1, 0)
 
         # The files haven't changed
         self.assertEqual(self.get_cache_mtimes(), mtimes)
@@ -773,6 +793,53 @@ class TestCache(TestCase):
         f = mod.renamed_function2
         self.assertPreciseEqual(f(2), 8)
 
+    def _test_pycache_fallback(self):
+        """
+        With a disabled __pycache__, test there is a working fallback
+        (e.g. on the user-wide cache dir)
+        """
+        mod = self.import_module()
+        f = mod.add_usecase
+        # Remove this function's cache files at the end, to avoid accumulation
+        # accross test calls.
+        self.addCleanup(shutil.rmtree, f.stats.cache_path, ignore_errors=True)
+
+        self.assertPreciseEqual(f(2, 3), 6)
+        # It's a cache miss since the file was copied to a new temp location
+        self.check_hits(f, 0, 1)
+
+        # Test re-use
+        mod2 = self.import_module()
+        f = mod2.add_usecase
+        self.assertPreciseEqual(f(2, 3), 6)
+        self.check_hits(f, 1, 0)
+
+        # The __pycache__ is empty (otherwise the test's preconditions
+        # wouldn't be met)
+        self.check_pycache(0)
+
+    @unittest.skipIf(os.name == "nt",
+                     "cannot easily make a directory read-only on Windows")
+    def test_non_creatable_pycache(self):
+        # Make it impossible to create the __pycache__ directory
+        old_perms = os.stat(self.tempdir).st_mode
+        os.chmod(self.tempdir, 0o500)
+        self.addCleanup(os.chmod, self.tempdir, old_perms)
+
+        self._test_pycache_fallback()
+
+    @unittest.skipIf(os.name == "nt",
+                     "cannot easily make a directory read-only on Windows")
+    def test_non_writable_pycache(self):
+        # Make it impossible to write to the __pycache__ directory
+        pycache = os.path.join(self.tempdir, '__pycache__')
+        os.mkdir(pycache)
+        old_perms = os.stat(pycache).st_mode
+        os.chmod(pycache, 0o500)
+        self.addCleanup(os.chmod, pycache, old_perms)
+
+        self._test_pycache_fallback()
+
 
 if __name__ == '__main__':
     unittest.main()
