@@ -10,7 +10,7 @@ import numpy as np
 
 import numba.unittest_support as unittest
 from numba import config, numpy_support, types
-from .support import TestCase, skip_on_numpy_16, tag
+from .support import TestCase, tag
 
 
 class TestFromDtype(TestCase):
@@ -81,7 +81,6 @@ class TestFromDtype(TestCase):
         check('a11', types.CharSeq(11))
         check('U12', types.UnicodeCharSeq(12))
 
-    @skip_on_numpy_16
     def check_datetime_types(self, letter, nb_class):
         def check(dtype, numba_type, code):
             tp = numpy_support.from_dtype(dtype)
@@ -177,14 +176,12 @@ class ValueTypingTestBase(object):
             # This ensures the unit hasn't been lost
             self.assertEqual(tp, nb_type(unit))
 
-    @skip_on_numpy_16
     def check_datetime_values(self, func):
         """
         Test *func*() with np.datetime64 values.
         """
         self._base_check_datetime_values(func, np.datetime64, types.NPDatetime)
 
-    @skip_on_numpy_16
     def check_timedelta_values(self, func):
         """
         Test *func*() with np.timedelta64 values.
