@@ -1,5 +1,6 @@
 from __future__ import print_function
 
+import collections
 import functools
 import math
 import os
@@ -41,6 +42,21 @@ def random_randrange2(a, b):
 def random_randrange3(a, b, c):
     return random.randrange(a, b, c)
 
+def numpy_choice1(a):
+    return np.random.choice(a)
+
+def numpy_choice2(a, size):
+    return np.random.choice(a, size=size)
+
+def numpy_choice3(a, size, replace):
+    return np.random.choice(a, size=size, replace=replace)
+
+def numpy_multinomial2(n, pvals):
+    return np.random.multinomial(n, pvals)
+
+def numpy_multinomial3(n, pvals, size):
+    return np.random.multinomial(n, pvals=pvals, size=size)
+
 
 def jit_with_args(name, argstring):
     code = """def func(%(argstring)s):
@@ -113,7 +129,20 @@ def py_f(r, num, denom):
             (py_chisquare(r, denom) * num))
 
 
-class TestInternals(TestCase):
+class BaseTest(TestCase):
+
+    def _follow_cpython(self, ptr, seed=2):
+        r = random.Random(seed)
+        _copy_py_state(r, ptr)
+        return r
+
+    def _follow_numpy(self, ptr, seed=2):
+        r = np.random.RandomState(seed)
+        _copy_np_state(r, ptr)
+        return r
+
+
+class TestInternals(BaseTest):
     """
     Test low-level internals of the implementation.
     """
@@ -181,27 +210,17 @@ class TestInternals(TestCase):
         self._check_perturb(py_state_ptr)
 
 
-class TestRandom(TestCase):
+class TestRandom(BaseTest):
 
     # NOTE: there may be cascading imprecision issues (e.g. between x87-using
     # C code and SSE-using LLVM code), which is especially brutal for some
     # iterative algorithms with sensitive exit conditions.
-    # Therefore we stick to hardcoded integers for seed values below.
+    # Therefore we stick to hardcoded integers for seed values.
 
     def setUp(self):
         # Make sure the PRNG is initialized before we set the seed
         random_init()
 
-    def _follow_cpython(self, ptr, seed=2):
-        r = random.Random(seed)
-        _copy_py_state(r, ptr)
-        return r
-
-    def _follow_numpy(self, ptr, seed=2):
-        r = np.random.RandomState(seed)
-        _copy_np_state(r, ptr)
-        return r
-
     def _check_random_seed(self, seedfunc, randomfunc):
         """
         Check seed()- and random()-like functions.
@@ -513,14 +532,7 @@ class TestRandom(TestCase):
         """
         # Our implementation follows Python 2.7+'s.
         r = self._follow_cpython(ptr)
-        if sys.version_info >= (2, 7):
-            self._check_dist(func, r.vonmisesvariate, [(0.5, 2.5)])
-        else:
-            # Sanity check
-            for i in range(10):
-                val = func(0.5, 2.5)
-                self.assertGreaterEqual(val, 0.0)
-                self.assertLess(val, 2 * math.pi)
+        self._check_dist(func, r.vonmisesvariate, [(0.5, 2.5)])
 
     def test_random_vonmisesvariate(self):
         self._check_vonmisesvariate(jit_binary("random.vonmisesvariate"),
@@ -606,8 +618,25 @@ class TestRandom(TestCase):
     @tag('important')
     def test_numpy_binomial(self):
         # We follow Numpy's algorithm up to n*p == 30
-        self._follow_numpy(np_state_ptr, 0)
         binomial = jit_binary("np.random.binomial")
+        r = self._follow_numpy(np_state_ptr, 0)
+        self._check_dist(binomial, r.binomial, [(18, 0.25)])
+        # Sanity check many values
+        for n in (100, 1000, 10000):
+            self.assertEqual(binomial(n, 0.0), 0)
+            self.assertEqual(binomial(n, 1.0), n)
+            for p in (0.0001, 0.1, 0.4, 0.49999, 0.5, 0.50001, 0.8, 0.9, 0.9999):
+                r = binomial(n, p)
+                if p > 0.5:
+                    r = n - r
+                    p = 1 - p
+                self.assertGreaterEqual(r, 0)
+                self.assertLessEqual(r, n)
+                expected = p * n
+                tol = 3 * n / math.sqrt(n)
+                self.assertGreaterEqual(r, expected - tol, (p, n, r))
+                self.assertLessEqual(r, expected + tol, (p, n, r))
+        # Invalid values
         self.assertRaises(ValueError, binomial, -1, 0.5)
         self.assertRaises(ValueError, binomial, 10, -0.1)
         self.assertRaises(ValueError, binomial, 10, 1.1)
@@ -805,15 +834,14 @@ class TestRandom(TestCase):
                 self.assertFalse(np.all(a == b))
                 self.assertEqual(sorted(a), sorted(b))
                 a = b
-        if sys.version_info >= (2, 7):
-            # Test with an arbitrary buffer-providing object
-            b = a.copy()
-            func(memoryview(b))
-            self.assertFalse(np.all(a == b))
-            self.assertEqual(sorted(a), sorted(b))
-            # Read-only object
-            with self.assertTypingError():
-                func(memoryview(b"xyz"))
+        # Test with an arbitrary buffer-providing object
+        b = a.copy()
+        func(memoryview(b))
+        self.assertFalse(np.all(a == b))
+        self.assertEqual(sorted(a), sorted(b))
+        # Read-only object
+        with self.assertTypingError():
+            func(memoryview(b"xyz"))
 
     @tag('important')
     def test_random_shuffle(self):
@@ -856,5 +884,362 @@ class TestRandom(TestCase):
         self._check_startup_randomness("numpy_normal", (1.0, 1.0))
 
 
+class TestRandomArrays(BaseTest):
+    """
+    Test array-producing variants of np.random.* functions.
+    """
+
+    def setUp(self):
+        # Make sure the PRNG is initialized before we set the seed
+        random_init()
+
+    def _compile_array_dist(self, funcname, nargs):
+        qualname = "np.random.%s" % (funcname,)
+        argstring = ', '.join('abcd'[:nargs])
+        return jit_with_args(qualname, argstring)
+
+    def _check_array_dist(self, funcname, scalar_args):
+        """
+        Check returning an array according to a given distribution.
+        """
+        cfunc = self._compile_array_dist(funcname, len(scalar_args) + 1)
+        r = self._follow_numpy(np_state_ptr)
+        pyfunc = getattr(r, funcname)
+        for size in (8, (2, 3)):
+            args = scalar_args + (size,)
+            expected = pyfunc(*args)
+            got = cfunc(*args)
+            # Numpy may return int32s where we return int64s, adjust
+            if (expected.dtype == np.dtype('int32')
+                and got.dtype == np.dtype('int64')):
+                expected = expected.astype(got.dtype)
+            self.assertPreciseEqual(expected, got, prec='double', ulps=5)
+
+    def test_numpy_randint(self):
+        cfunc = self._compile_array_dist("randint", 3)
+        low, high = 1000, 10000
+        size = (30, 30)
+        res = cfunc(low, high, size)
+        self.assertIsInstance(res, np.ndarray)
+        self.assertEqual(res.shape, size)
+        self.assertIn(res.dtype, (np.dtype('int32'), np.dtype('int64')))
+        self.assertTrue(np.all(res >= low))
+        self.assertTrue(np.all(res < high))
+        # Crude statistical tests
+        mean = (low + high) / 2
+        tol = (high - low) / 20
+        self.assertGreaterEqual(res.mean(), mean - tol)
+        self.assertLessEqual(res.mean(), mean + tol)
+
+    def test_numpy_random_random(self):
+        cfunc = self._compile_array_dist("random", 1)
+        size = (30, 30)
+        res = cfunc(size)
+        self.assertIsInstance(res, np.ndarray)
+        self.assertEqual(res.shape, size)
+        self.assertEqual(res.dtype, np.dtype('float64'))
+        # Results are within expected bounds
+        self.assertTrue(np.all(res >= 0.0))
+        self.assertTrue(np.all(res < 1.0))
+        # Crude statistical tests
+        self.assertTrue(np.any(res <= 0.1))
+        self.assertTrue(np.any(res >= 0.9))
+        mean = res.mean()
+        self.assertGreaterEqual(mean, 0.45)
+        self.assertLessEqual(mean, 0.55)
+
+    # Sanity-check various distributions.  For convenience, we only check
+    # those distributions that produce the exact same values as Numpy's.
+
+    def test_numpy_binomial(self):
+        self._check_array_dist("binomial", (20, 0.5))
+
+    def test_numpy_exponential(self):
+        self._check_array_dist("exponential", (1.5,))
+
+    def test_numpy_gumbel(self):
+        self._check_array_dist("gumbel", (1.5, 0.5))
+
+    def test_numpy_laplace(self):
+        self._check_array_dist("laplace", (1.5, 0.5))
+
+    def test_numpy_logistic(self):
+        self._check_array_dist("logistic", (1.5, 0.5))
+
+    def test_numpy_lognormal(self):
+        self._check_array_dist("lognormal", (1.5, 2.0))
+
+    def test_numpy_logseries(self):
+        self._check_array_dist("logseries", (0.8,))
+
+    @tag('important')
+    def test_numpy_normal(self):
+        self._check_array_dist("normal", (0.5, 2.0))
+
+    def test_numpy_poisson(self):
+        self._check_array_dist("poisson", (0.8,))
+
+    def test_numpy_power(self):
+        self._check_array_dist("power", (0.8,))
+
+    def test_numpy_rayleigh(self):
+        self._check_array_dist("rayleigh", (0.8,))
+
+    def test_numpy_standard_cauchy(self):
+        self._check_array_dist("standard_cauchy", ())
+
+    def test_numpy_standard_exponential(self):
+        self._check_array_dist("standard_exponential", ())
+
+    def test_numpy_standard_normal(self):
+        self._check_array_dist("standard_normal", ())
+
+    def test_numpy_uniform(self):
+        self._check_array_dist("uniform", (0.1, 0.4))
+
+    def test_numpy_wald(self):
+        self._check_array_dist("wald", (0.1, 0.4))
+
+    def test_numpy_zipf(self):
+        self._check_array_dist("zipf", (2.5,))
+
+
+class TestRandomChoice(BaseTest):
+    """
+    Test np.random.choice.
+    """
+
+    def setUp(self):
+        # Make sure the PRNG is initialized before we set the seed
+        random_init()
+
+    def _check_results(self, pop, res, replace=True):
+        """
+        Check basic expectations about a batch of samples.
+        """
+        spop = set(pop)
+        sres = set(res)
+        # All results are in the population
+        self.assertLessEqual(sres, spop)
+        # Sorted results are unlikely
+        self.assertNotEqual(sorted(res), list(res))
+        if replace:
+            # Duplicates are likely
+            self.assertLess(len(sres), len(res), res)
+        else:
+            # No duplicates
+            self.assertEqual(len(sres), len(res), res)
+
+    def _check_dist(self, pop, samples):
+        """
+        Check distribution of some samples.
+        """
+        # Sanity check that we have enough samples
+        self.assertGreaterEqual(len(samples), len(pop) * 100)
+        # Check equidistribution of samples
+        expected_frequency = len(samples) / len(pop)
+        c = collections.Counter(samples)
+        for value in pop:
+            n = c[value]
+            self.assertGreaterEqual(n, expected_frequency * 0.5)
+            self.assertLessEqual(n, expected_frequency * 2.0)
+
+    def _accumulate_array_results(self, func, nresults):
+        """
+        Accumulate array results produced by *func* until they reach
+        *nresults* elements.
+        """
+        res = []
+        while len(res) < nresults:
+            res += list(func().flat)
+        return res[:nresults]
+
+    def _check_choice_1(self, a, pop):
+        """
+        Check choice(a) against pop.
+        """
+        cfunc = jit(nopython=True)(numpy_choice1)
+        n = len(pop)
+        res = [cfunc(a) for i in range(n)]
+        self._check_results(pop, res)
+        dist = [cfunc(a) for i in range(n * 100)]
+        self._check_dist(pop, dist)
+
+    def test_choice_scalar_1(self):
+        """
+        Test choice(int)
+        """
+        n = 50
+        pop = list(range(n))
+        self._check_choice_1(n, pop)
+
+    def test_choice_array_1(self):
+        """
+        Test choice(array)
+        """
+        pop = np.arange(50) * 2 + 100
+        self._check_choice_1(pop, pop)
+
+    def _check_array_results(self, func, pop, replace=True):
+        """
+        Check array results produced by *func* and their distribution.
+        """
+        n = len(pop)
+        res = list(func().flat)
+        self._check_results(pop, res, replace)
+        dist = self._accumulate_array_results(func, n * 100)
+        self._check_dist(pop, dist)
+
+    def _check_choice_2(self, a, pop):
+        """
+        Check choice(a, size) against pop.
+        """
+        cfunc = jit(nopython=True)(numpy_choice2)
+        n = len(pop)
+        # Final sizes should be large enough, so as to stress
+        # replacement
+        sizes = [n - 10, (3, (n - 1) // 3), n * 10]
+
+        for size in sizes:
+            # Check result shape
+            res = cfunc(a, size)
+            expected_shape = size if isinstance(size, tuple) else (size,)
+            self.assertEqual(res.shape, expected_shape)
+            # Check results and their distribution
+            self._check_array_results(lambda: cfunc(a, size), pop)
+
+    def test_choice_scalar_2(self):
+        """
+        Test choice(int, size)
+        """
+        n = 50
+        pop = np.arange(n)
+        self._check_choice_2(n, pop)
+
+    def test_choice_array_2(self):
+        """
+        Test choice(array, size)
+        """
+        pop = np.arange(50) * 2 + 100
+        self._check_choice_2(pop, pop)
+
+    def _check_choice_3(self, a, pop):
+        """
+        Check choice(a, size, replace) against pop.
+        """
+        cfunc = jit(nopython=True)(numpy_choice3)
+        n = len(pop)
+        # Final sizes should be close but slightly <= n, so as to stress
+        # replacement (or not)
+        sizes = [n - 10, (3, (n - 1) // 3)]
+        replaces = [True, False]
+
+        # Check result shapes
+        for size in sizes:
+            for replace in [True, False]:
+                res = cfunc(a, size, replace)
+                expected_shape = size if isinstance(size, tuple) else (size,)
+                self.assertEqual(res.shape, expected_shape)
+
+        # Check results for replace=True
+        for size in sizes:
+            self._check_array_results(lambda: cfunc(a, size, True), pop)
+        # Check results for replace=False
+        for size in sizes:
+            self._check_array_results(lambda: cfunc(a, size, False), pop, False)
+
+        # Can't ask for more samples than population size with replace=False
+        for size in [n + 1, (3, n // 3 + 1)]:
+            with self.assertRaises(ValueError):
+                cfunc(a, size, False)
+
+    def test_choice_scalar_3(self):
+        """
+        Test choice(int, size, replace)
+        """
+        n = 50
+        pop = np.arange(n)
+        self._check_choice_3(n, pop)
+
+    def test_choice_array_3(self):
+        """
+        Test choice(array, size, replace)
+        """
+        pop = np.arange(50) * 2 + 100
+        self._check_choice_3(pop, pop)
+
+
+class TestRandomMultinomial(BaseTest):
+    """
+    Test np.random.multinomial.
+    """
+    # A biased dice
+    pvals = np.array([1, 1, 1, 2, 3, 1], dtype=np.float64)
+    pvals /= pvals.sum()
+
+    def setUp(self):
+        # Make sure the PRNG is initialized before we set the seed
+        random_init()
+
+    def _check_sample(self, n, pvals, sample):
+        """
+        Check distribution of some samples.
+        """
+        self.assertIsInstance(sample, np.ndarray)
+        self.assertEqual(sample.shape, (len(pvals),))
+        self.assertIn(sample.dtype, (np.dtype('int32'), np.dtype('int64')))
+        # Statistical properties
+        self.assertEqual(sample.sum(), n)
+        for p, nexp in zip(pvals, sample):
+            self.assertGreaterEqual(nexp, 0)
+            self.assertLessEqual(nexp, n)
+            pexp = float(nexp) / n
+            self.assertGreaterEqual(pexp, p * 0.5)
+            self.assertLessEqual(pexp, p * 2.0)
+
+    def test_multinomial_2(self):
+        """
+        Test multinomial(n, pvals)
+        """
+        cfunc = jit(nopython=True)(numpy_multinomial2)
+        n, pvals = 1000, self.pvals
+        res = cfunc(n, pvals)
+        self._check_sample(n, pvals, res)
+        # pvals as list
+        pvals = list(pvals)
+        res = cfunc(n, pvals)
+        self._check_sample(n, pvals, res)
+        # A case with extreme probabilities
+        n = 1000000
+        pvals = np.array([1, 0, n // 100, 1], dtype=np.float64)
+        pvals /= pvals.sum()
+        res = cfunc(n, pvals)
+        self._check_sample(n, pvals, res)
+
+    def test_multinomial_3_int(self):
+        """
+        Test multinomial(n, pvals, size: int)
+        """
+        cfunc = jit(nopython=True)(numpy_multinomial3)
+        n, pvals = 1000, self.pvals
+        k = 10
+        res = cfunc(n, pvals, k)
+        self.assertEqual(res.shape[0], k)
+        for sample in res:
+            self._check_sample(n, pvals, sample)
+
+    def test_multinomial_3_tuple(self):
+        """
+        Test multinomial(n, pvals, size: tuple)
+        """
+        cfunc = jit(nopython=True)(numpy_multinomial3)
+        n, pvals = 1000, self.pvals
+        k = (3, 4)
+        res = cfunc(n, pvals, k)
+        self.assertEqual(res.shape[:-1], k)
+        for sample in res.reshape((-1, res.shape[-1])):
+            self._check_sample(n, pvals, sample)
+
+
 if __name__ == "__main__":
     unittest.main()
