@@ -8,6 +8,7 @@ from numba import unittest_support as unittest
 from numba.compiler import compile_isolated
 from numba.funcdesc import transform_arg_name
 from numba.utils import IS_PY3
+from .support import tag
 
 
 def get_a(ary, i):
@@ -25,6 +26,10 @@ def make_getitem(item):
         return ary[i][item]
     return get_xx
 
+# Issue #1664: constant index lookup should fall back to regular getitem
+def get_zero_a(ary, _unused):
+    return ary[0].a
+
 getitem_a = make_getitem('a')
 getitem_b = make_getitem('b')
 getitem_c = make_getitem('c')
@@ -38,6 +43,9 @@ def get_b_subarray(ary, i):
 def get_c_subarray(ary, i):
     return ary.c[i]
 
+def get_a_zero(ary, _unused):
+    return ary.a[0]
+
 def make_getitem_subarray(item):
     # This also exercises constant lookup from a closure variable
     def get_xx_subarray(ary, i):
@@ -303,11 +311,14 @@ class TestRecordDtype(unittest.TestCase):
         for i in range(self.refsample1d.size):
             self.assertEqual(pyfunc(self.refsample1d, i), cfunc(self.nbsample1d, i))
 
+    @tag('important')
     def test_get_a(self):
         self._test_get_equal(get_a)
         self._test_get_equal(get_a_subarray)
         self._test_get_equal(getitem_a)
         self._test_get_equal(getitem_a_subarray)
+        self._test_get_equal(get_a_zero)
+        self._test_get_equal(get_zero_a)
 
     def test_get_b(self):
         self._test_get_equal(get_b)
@@ -367,6 +378,7 @@ class TestRecordDtype(unittest.TestCase):
             # Match the entire array to ensure no memory corruption
             self.assertTrue(np.all(expect == got))
 
+    @tag('important')
     def test_set_a(self):
         def check(pyfunc):
             self._test_set_equal(pyfunc, 3.1415, types.float64)
@@ -387,6 +399,7 @@ class TestRecordDtype(unittest.TestCase):
         check(setitem_b)
         check(setitem_b_subarray)
 
+    @tag('important')
     def test_set_c(self):
         def check(pyfunc):
             self._test_set_equal(pyfunc, 43j, types.complex64)
@@ -397,6 +410,7 @@ class TestRecordDtype(unittest.TestCase):
         check(setitem_c)
         check(setitem_c_subarray)
 
+    @tag('important')
     def test_set_record(self):
         pyfunc = set_record
         rec = numpy_support.from_dtype(recordtype)
@@ -415,7 +429,6 @@ class TestRecordDtype(unittest.TestCase):
             self.assertEqual(got[i], got[j])
             self.assertTrue(np.all(expect == got))
 
-
     def _test_record_args(self, revargs):
         """
         Testing scalar record value as argument
@@ -525,6 +538,7 @@ class TestRecordDtype(unittest.TestCase):
         np.testing.assert_equal(expected, nbval)
 
 
+    @tag('important')
     def test_record_write_2d_array(self):
         '''
         Test writing to a 2D array within a structured type
@@ -558,6 +572,7 @@ class TestRecordDtype(unittest.TestCase):
         np.testing.assert_equal(res, nbval[0].h[1])
 
 
+    @tag('important')
     def test_record_read_2d_array(self):
         '''
         Test reading from a 2D array within a structured type
@@ -579,6 +594,7 @@ class TestRecordDtype(unittest.TestCase):
         np.testing.assert_equal(res, nbval[0].j[1, 0])
 
 
+    @tag('important')
     def test_record_return(self):
         """
         Testing scalar record value as return value.
@@ -647,9 +663,11 @@ class TestRecordDtype(unittest.TestCase):
         cfunc = self.get_cfunc(pyfunc, (nbrecord,))
         self.assertEqual(cfunc(rec), pyfunc(rec))
 
+
 def _get_cfunc_nopython(pyfunc, argspec):
     return jit(argspec, nopython=True)(pyfunc)
 
+
 class TestRecordDtypeWithDispatcher(TestRecordDtype):
     '''
     Same as TestRecordDtype, but stressing the Dispatcher's type dispatch
@@ -660,6 +678,7 @@ class TestRecordDtypeWithDispatcher(TestRecordDtype):
     def get_cfunc(self, pyfunc, argspec):
         return _get_cfunc_nopython(pyfunc, argspec)
 
+
 class TestRecordDtypeWithStructArrays(TestRecordDtype):
     '''
     Same as TestRecordDtype, but using structured arrays instead of recarrays.
