@@ -4,9 +4,9 @@ import itertools
 
 import numpy as np
 
-from numba import utils
+from numba import jit, utils
 from numba import unittest_support as unittest
-from .support import TestCase, skip_on_numpy_16
+from .support import TestCase, forbid_codegen
 
 DBL_EPSILON = 2**-52
 FLT_EPSILON = 2**-23
@@ -274,7 +274,6 @@ class TestAssertPreciseEqual(TestCase):
         self.eq(a, b, prec='double')
         self.ne(a, c, prec='double')
 
-    @skip_on_numpy_16
     def test_npdatetime(self):
         a = np.datetime64('1900', 'Y')
         b = np.datetime64('1900', 'Y')
@@ -286,7 +285,6 @@ class TestAssertPreciseEqual(TestCase):
         # Different value
         self.ne(a, d)
 
-    @skip_on_numpy_16
     def test_nptimedelta(self):
         a = np.timedelta64(1, 'h')
         b = np.timedelta64(1, 'h')
@@ -314,6 +312,19 @@ class TestMisc(TestCase):
                 l.append(y)
         self.assertIn("66", str(cm.exception))
 
+    def test_forbid_codegen(self):
+        """
+        Test that forbid_codegen() prevents code generation using the @jit
+        decorator.
+        """
+        def f():
+            return 1
+        with forbid_codegen():
+            with self.assertRaises(RuntimeError) as raises:
+                cfunc = jit(nopython=True)(f)
+                cfunc()
+        self.assertIn("codegen forbidden by test case", str(raises.exception))
+
 
 if __name__ == '__main__':
     unittest.main()
