@@ -495,6 +495,16 @@ class StaticGetItemTuple(AbstractTemplate):
             return types.BaseTuple.from_types(tup.types[idx])
 
 
+# Generic implementation for "not in"
+
+@infer
+class GenericNotIn(AbstractTemplate):
+    key = "not in"
+
+    def generic(self, args, kws):
+        return self.context.resolve_function_type("in", args, kws)
+
+
 #-------------------------------------------------------------------------------
 
 @infer_getattr
@@ -540,6 +550,12 @@ class BooleanAttribute(AttributeTemplate):
     def resolve___class__(self, ty):
         return types.NumberClass(ty)
 
+    @bound_function("number.item")
+    def resolve_item(self, ty, args, kws):
+        assert not kws
+        if not args:
+            return signature(ty)
+
 
 @infer_getattr
 class NumberAttribute(AttributeTemplate):
@@ -560,6 +576,12 @@ class NumberAttribute(AttributeTemplate):
         assert not kws
         return signature(ty)
 
+    @bound_function("number.item")
+    def resolve_item(self, ty, args, kws):
+        assert not kws
+        if not args:
+            return signature(ty)
+
 
 @infer_getattr
 class SliceAttribute(AttributeTemplate):
@@ -669,6 +691,16 @@ class Round(ConcreteTemplate):
     ]
 
 
+@infer_global(hash)
+class Hash(AbstractTemplate):
+
+    def generic(self, args, kws):
+        assert not kws
+        arg, = args
+        if isinstance(arg, types.Hashable):
+            return signature(types.intp, *args)
+
+
 #------------------------------------------------------------------------------
 
 
@@ -776,6 +808,29 @@ class Zip(AbstractTemplate):
             zip_type = types.ZipType(args)
             return signature(zip_type, *args)
 
+
+@infer_global(iter)
+class Iter(AbstractTemplate):
+
+    def generic(self, args, kws):
+        assert not kws
+        if len(args) == 1:
+            it = args[0]
+            if isinstance(it, types.IterableType):
+                return signature(it.iterator_type, *args)
+
+
+@infer_global(next)
+class Next(AbstractTemplate):
+
+    def generic(self, args, kws):
+        assert not kws
+        if len(args) == 1:
+            it = args[0]
+            if isinstance(it, types.IteratorType):
+                return signature(it.yield_type, *args)
+
+
 #------------------------------------------------------------------------------
 
 @infer_global(type)
