@@ -82,9 +82,8 @@ class BaseContext(object):
                 defns.extend(getattr(tpl, 'cases', []))
 
         else:
-            msg = "No type info available for {func} as a callable."
+            msg = "No type info available for {func!r} as a callable."
             desc.append(msg.format(func=func))
-            return desc
 
         if defns:
             desc = ['Known signatures:']
@@ -465,14 +464,14 @@ class BaseContext(object):
         unified = typelist[0]
         for tp in typelist[1:]:
             unified = self.unify_pairs(unified, tp)
-            if unified is types.pyobject:
+            if unified is None:
                 break
         return unified
 
     def unify_pairs(self, first, second):
         """
         Try to unify the two given types.  A third type is returned,
-        or pyobject in case of failure.
+        or None in case of failure.
         """
         if first == second:
             return first
@@ -503,14 +502,14 @@ class BaseContext(object):
             return first
 
         # Cannot unify
-        return types.pyobject
+        return None
 
 
 class Context(BaseContext):
 
     def load_additional_registries(self):
         from . import (cffi_utils, cmathdecl, listdecl, mathdecl,
-                       npydecl, operatordecl, randomdecl)
+                       npydecl, operatordecl, randomdecl, setdecl)
         self.install_registry(cffi_utils.registry)
         self.install_registry(cmathdecl.registry)
         self.install_registry(listdecl.registry)
@@ -518,3 +517,4 @@ class Context(BaseContext):
         self.install_registry(npydecl.registry)
         self.install_registry(operatordecl.registry)
         self.install_registry(randomdecl.registry)
+        self.install_registry(setdecl.registry)
