@@ -52,12 +52,6 @@ def is_ctypes_funcptr(obj):
         return hasattr(obj, 'argtypes') and hasattr(obj, 'restype')
 
 
-@typeof_impl.register(ctypes._CFuncPtr)
-def typeof_ctypes_function(val, c):
-    if is_ctypes_funcptr(val):
-        return make_function_type(val)
-
-
 def get_pointer(ctypes_func):
     """
     Get a pointer to the underlying function for a ctypes function as an
