@@ -365,7 +365,7 @@ def _numpy_redirect(fname):
                dict(key=numpy_function, method_name=fname))
     infer_global(numpy_function, types.Function(cls))
 
-for func in ['min', 'max', 'sum', 'prod', 'mean', 'median', 'var', 'std',
+for func in ['min', 'max', 'sum', 'prod', 'mean', 'var', 'std',
              'cumsum', 'cumprod', 'argmin', 'argmax', 'nonzero', 'ravel']:
     _numpy_redirect(func)
 
@@ -433,7 +433,7 @@ def _parse_nested_sequence(context, typ):
                                   % (typ,))
             dtypes.append(dtype)
         dtype = context.unify_types(*dtypes)
-        if dtype is types.pyobject:
+        if dtype is None:
             raise TypingError("cannot convert %r to a homogenous type")
         return n + 1, dtype
     else:
@@ -514,9 +514,7 @@ class NdConstructorLike(CallableTemplate):
         return typer
 
 
-if numpy_version >= (1, 7):
-    # In Numpy 1.6, ones_like() was a ufunc and had a different signature.
-    infer_global(numpy.ones_like)(NdConstructorLike)
+infer_global(numpy.ones_like)(NdConstructorLike)
 
 
 if numpy_version >= (1, 8):
@@ -809,6 +807,25 @@ class NdEnumerate(AbstractTemplate):
             return signature(enumerate_type, *args)
 
 
+@infer_global(numpy.nditer)
+class NdIter(AbstractTemplate):
+
+    def generic(self, args, kws):
+        assert not kws
+        if len(args) != 1:
+            return
+        arrays, = args
+
+        if isinstance(arrays, types.BaseTuple):
+            if not arrays:
+                return
+            arrays = list(arrays)
+        else:
+            arrays = [arrays]
+        nditerty = types.NumpyNdIterType(arrays)
+        return signature(nditerty, *args)
+
+
 @infer_global(numpy.ndindex)
 class NdIndex(AbstractTemplate):
 
